'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deepMapParentsOutwards = exports.deepMapLeavesOutwards = exports.deepMapOutwards = exports.deepMapParents = exports.deepMapLeaves = exports.deepMap = undefined;

var _immutable = require('immutable');

var _utils = require('./utils');

/**
 * @module map
 */

var mapRecursive = function mapRecursive(self, childPath, mapper, tree, options) {
  var nodePath = arguments.length <= 5 || arguments[5] === undefined ? (0, _immutable.List)() : arguments[5];

  var mapSelf = function mapSelf(self) {
    return mapper(self, nodePath, tree);
  };

  if ((0, _utils.isLeaf)(self, childPath)) {
    return options.onlyParents ? self : mapSelf(self);
  }

  var mappedChildren = self.getIn(childPath).map(function (kid, key) {
    return mapRecursive(kid, childPath, mapper, tree, options, nodePath.push(key));
  });

  var selfWithMappedChildren = childPath.isEmpty() ? self.merge(mappedChildren) : self.setIn(childPath, mappedChildren);

  return options.onlyLeaves ? selfWithMappedChildren : mapSelf(selfWithMappedChildren);
};

var mapRecursiveOutwards = function mapRecursiveOutwards(self, childPath, mapper, tree, options) {
  var nodePath = arguments.length <= 5 || arguments[5] === undefined ? (0, _immutable.List)() : arguments[5];

  if (options.onlyParents && (0, _utils.isLeaf)(self, childPath)) {
    return self;
  }

  var mappedSelf;
  if (options.onlyLeaves) {
    mappedSelf = self;
    if ((0, _utils.isLeaf)(mappedSelf, childPath)) {
      return mapper(self, nodePath, tree);
    }
  } else {
    mappedSelf = mapper(self, nodePath, tree);
    if ((0, _utils.isLeaf)(mappedSelf, childPath)) {
      return mappedSelf;
    }
  }

  return mappedSelf.updateIn(childPath, function (children) {
    return children.map(function (kid, key) {
      return mapRecursiveOutwards(kid, childPath, mapper, tree, options, nodePath.push(key));
    });
  });
};

/**
 * Once fully applied, this iterates through all nodes in the provided tree, passing them all through a `mapper` function.
 * Nodes are processed branch by branch in the order that Immutable maps through the child iterables, inward from leaves to the root node.
 * Sibling nodes are processed in the order that Immutable iterates through collections.
 *
 * `TODO: note about care with child collections, note about using as reduce`
 * 
 * @param {Mapper} mapper The function to be called for every node in the tree, the results of which will be used to create the modified tree.
 * @param {ChildPath} [childPath=null] An `Array` or `List` of keys indicating where to find each node's children from within each node.
 * @return {InputFunction} A partially applied function which accepts a single tree `Iterable`, and returns the modified tree `Iterable`.
 */

function deepMap(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return function (tree) {
    return mapRecursive(tree, (0, _utils.asList)(childPath), mapper, tree, {});
  };
}

/**
 * Once fully applied, this iterates through the leaf nodes in the provided tree, passing them all through a `mapper` function. Leaf nodes are nodes that have no child nodes.
 * Nodes are processed branch by branch, in the order that Immutable maps through the child iterables.
 *
 * @param {Mapper} mapper The function to be called for every leaf node in the tree, the results of which will be used to create the modified tree.
 * @param {ChildPath} [childPath=null] An `Array` or `List` of keys indicating where to find each node's children from within each node.
 * @return {InputFunction} A partially applied function which accepts a single tree `Iterable`, and returns the modified tree `Iterable`.
 */

function deepMapLeaves(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return function (tree) {
    return mapRecursive(tree, (0, _utils.asList)(childPath), mapper, tree, { onlyLeaves: true });
  };
}

/**
 * Once fully applied, this iterates through the parent nodes in the provided tree, passing them all through a `mapper` function. Parent nodes are nodes that have child nodes.
 * Nodes are processed branch by branch in the order that Immutable maps through the child iterables, inward from leaves to the root node.
 *
 * @param {Mapper} mapper The function to be called for every parent node in the tree.
 * the results of which will be used to create the modified iterable.
 * @param {ChildPath} [childPath=null] An `Array` or `List` of keys indicating where to find each node's children from within each node.
 * @return {InputFunction} A partially applied function which accepts a single tree `Iterable`, and returns the modified tree `Iterable`.
 */

function deepMapParents(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return function (tree) {
    return mapRecursive(tree, (0, _utils.asList)(childPath), mapper, tree, { onlyParents: true });
  };
}

/**
 * Once fully applied, this iterates through all nodes in the provided tree, passing them all through a `mapper` function.
 * Nodes are processed branch by branch in the order that Immutable maps through the child iterables, outwards from the root node to the leaf nodes.
 * Sibling nodes are processed in the order that Immutable iterates through collections.
 *
 * Because this moves from the root node outwards this function behaves quite differently to the other deep map functions.
 *
 * - If you modify a node's children, then those children will already be modified by the time they are called by the `mapper` function.
 * - Because of the above, if a node removes or renames its own children iterable then those children will not be passed through the `mapper` function.
 *
 * @param {Mapper} mapper The function to be called for every node in the tree, the results of which will be used to create the modified tree.
 * @param {ChildPath} [childPath=null] An `Array` or `List` of keys indicating where to find each node's children from within each node.
 * @return {InputFunction} A partially applied function which accepts a single tree `Iterable`, and returns the modified tree `Iterable`.
 */

function deepMapOutwards(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return function (tree) {
    return mapRecursiveOutwards(tree, (0, _utils.asList)(childPath), mapper, tree, {});
  };
}

/**
 * Once fully applied, this iterates through the leaf nodes in the provided tree, passing them all through a `mapper` function.
 * Nodes are processed branch by branch in the order that Immutable maps through the child iterables, outwards from the root node to the leaf nodes.
 * Sibling nodes are processed in the order that Immutable iterates through collections.
 *
 * @param {Mapper} mapper The function to be called for every leaf node in the tree, the results of which will be used to create the modified tree.
 * @param {ChildPath} [childPath=null] An `Array` or `List` of keys indicating where to find each node's children from within each node.
 * @return {InputFunction} A partially applied function which accepts a single tree `Iterable`, and returns the modified tree `Iterable`.
 */

function deepMapLeavesOutwards(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return function (tree) {
    return mapRecursiveOutwards(tree, (0, _utils.asList)(childPath), mapper, tree, { onlyLeaves: true });
  };
}

/**
 * Once fully applied, this iterates through the parent nodes in the provided tree, passing them all through a `mapper` function.
 * Nodes are processed branch by branch in the order that Immutable maps through the child iterables, outwards from the root node to the leaf nodes.
 * Sibling nodes are processed in the order that Immutable iterates through collections.
 *
 * Because this moves from the root node outwards this function behaves quite differently to the other deep map functions.
 *
 * - If you modify a node's children, then those children will already be modified by the time they are called by the `mapper` function.
 * - Because of the above, if a node removes or renames its own children iterable then those children will not be passed through the `mapper` function.
 *
 * @param {Mapper} mapper The function to be called for every parent node in the tree, the results of which will be used to create the modified tree.
 * @param {ChildPath} [childPath=null] An `Array` or `List` of keys indicating where to find each node's children from within each node.
 * @return {InputFunction} A partially applied function which accepts a single tree `Iterable`, and returns the modified tree `Iterable`.
 */

function deepMapParentsOutwards(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return function (tree) {
    return mapRecursiveOutwards(tree, (0, _utils.asList)(childPath), mapper, tree, { onlyParents: true });
  };
}

exports.deepMap = deepMap;
exports.deepMapLeaves = deepMapLeaves;
exports.deepMapParents = deepMapParents;
exports.deepMapOutwards = deepMapOutwards;
exports.deepMapLeavesOutwards = deepMapLeavesOutwards;
exports.deepMapParentsOutwards = deepMapParentsOutwards;